@@ -7,6 +7,14 @@ namespace Tomboy
 {
 	public class NoteRecentChanges : ForcedPresentWindow
 	{
+
+		private enum Target
+		{
+			Text,
+			Uri,
+			Path,
+		}
+
 		NoteManager manager;
 
 		Gtk.MenuBar menu_bar;
@@ -289,13 +297,16 @@ namespace Tomboy
 			new Gtk.TargetEntry [] {
 				new Gtk.TargetEntry ("STRING",
 				Gtk.TargetFlags.App,
-				0),
+				(uint) Target.Text),
 				new Gtk.TargetEntry ("text/plain",
 				Gtk.TargetFlags.App,
-				0),
+				(uint) Target.Text),
 				new Gtk.TargetEntry ("text/uri-list",
 				Gtk.TargetFlags.App,
-				1),
+				(uint) Target.Uri),
+				new Gtk.TargetEntry ("text/path-list",
+				Gtk.TargetFlags.App,
+				(uint) Target.Path),
 			};
 
 			tree = new RecentTreeView ();
@@ -662,17 +673,21 @@ namespace Tomboy
 				return;
 
 			string uris = string.Empty;
+			string paths = string.Empty;
 			foreach (Note note in selected_notes) {
-				if (uris != string.Empty)
-					uris += "\n";
-				uris += note.Uri;
+				uris += note.Uri + "\r\n";
+				paths += "file://" + note.FilePath + "\r\n";
 			}
 
-			// FIXME: Gtk.SelectionData has no way to get the
-			//	requested target.
-			args.SelectionData.Set (Gdk.Atom.Intern ("text/uri-list", false),
-								8,
-								Encoding.UTF8.GetBytes (uris));
+			if(args.Info == (uint) Target.Path)
+				args.SelectionData.Set (Gdk.Atom.Intern ("text/path-list", false),
+							8,
+							Encoding.UTF8.GetBytes (paths));
+
+			else
+				args.SelectionData.Set (Gdk.Atom.Intern ("text/uri-list", false),
+							8,
+							Encoding.UTF8.GetBytes (uris));
 
 			if (selected_notes.Count == 1)
 				args.SelectionData.Text = selected_notes [0].Title;
@@ -682,7 +697,8 @@ namespace Tomboy
 
 		void OnNotebookRowEdited (object sender, Gtk.EditedArgs args)
 		{
-			if (Notebooks.NotebookManager.NotebookExists (args.NewText))
+			if (Notebooks.NotebookManager.NotebookExists (args.NewText) ||
+			    string.IsNullOrEmpty (args.NewText))
 				return;
 			var oldNotebook = GetSelectedNotebook ();
 			if (oldNotebook is Notebooks.SpecialNotebook)
@@ -838,10 +854,18 @@ namespace Tomboy
 			if (x == 0 && y == 0)
 				pos_menu_func = PositionContextMenu;
 
-			menu.Popup (null, null,
-				    pos_menu_func,
-				    0,
-				    Gtk.Global.CurrentEventTime);
+			try {
+				menu.Popup (null, null,
+					    pos_menu_func,
+					    0,
+					    Gtk.Global.CurrentEventTime);
+			} catch {
+				Logger.Debug ("Menu popup failed with custom MenuPositionFunc; trying again without");
+				menu.Popup (null, null,
+					    null,
+					    0,
+					    Gtk.Global.CurrentEventTime);
+			}
 		}
 
 		// This is needed for when the user opens
